@@ -1,5 +1,5 @@
 import pickle
-from typing import Hashable, List
+from typing import Any, Hashable, List, Union
 
 import torch
 from quaterion_models.encoders import Encoder
@@ -23,6 +23,7 @@ class InMemoryCacheEncoder(CacheEncoder):
     ):
         super().__init__(encoder)
         self._cache = None
+        self._meta_cache = {}
         self._offset_map = {}
         self._cache_type = cache_type
         self._original_device = None
@@ -53,6 +54,16 @@ class InMemoryCacheEncoder(CacheEncoder):
 
         return embeddings
 
+    def cache_extract_meta(self, batch: Union[tuple, List[Hashable]]) -> List[dict]:
+        if isinstance(batch, tuple):
+            # Cache filling phase
+            _keys, features = batch
+            return self._encoder.get_meta_extractor()(features)
+        else:
+            # Assume training phase.
+            # Only keys are provided here
+            return [self._meta_cache[key] for key in batch]
+
     def get_collate_fn(self) -> "CollateFnType":
         """Provides function that converts raw data batch into suitable input.
 
@@ -65,7 +76,9 @@ class InMemoryCacheEncoder(CacheEncoder):
     def is_filled(self) -> bool:
         return self._cache is not None
 
-    def fill_cache(self, keys: List[Hashable], data: "TensorInterchange") -> None:
+    def fill_cache(
+        self, keys: List[Hashable], data: "TensorInterchange", meta: List[Any]
+    ) -> None:
         embeddings = self._encoder(data)
         if self.cache_type == CacheType.CPU:
             embeddings = embeddings.to("cpu")
@@ -76,6 +89,9 @@ class InMemoryCacheEncoder(CacheEncoder):
             self._offset_map[key] = len(self._offset_map)
         self._tmp.append(embeddings)
 
+        for key, meta_item in zip(keys, meta):
+            self._meta_cache[key] = meta_item
+
     def finish_fill(self):
         self._cache = torch.cat(self._tmp)
         self._tmp = []
@@ -83,14 +99,18 @@ class InMemoryCacheEncoder(CacheEncoder):
     def reset_cache(self) -> None:
         """Resets cache."""
         self._cache = None
+        self._meta_cache = {}
         self._offset_map = {}
         self._tmp = []
 
     def save_cache(self, path):
-        pickle.dump([self._cache.to("cpu"), self._offset_map], open(path, "wb"))
+        pickle.dump(
+            [self._cache.to("cpu"), self._offset_map, self._meta_cache],
+            open(path, "wb"),
+        )
 
     def load_cache(self, path):
-        self._cache, self._offset_map = pickle.load(open(path, "rb"))
+        self._cache, self._offset_map, self._meta_cache = pickle.load(open(path, "rb"))
         if self.cache_type != CacheType.CPU:
             device = self._encoder_device()
             self._cache = self._cache.to(device)
